WELCOME_KB = [
    ("j", "down"),
    ("k", "up"),
    ("enter", "confirm"),
    ("ctrl+c", "exit"),
    ("f1", "help"),
]

FORMS_KB = [
    ("ctrl+j", "down"),
    ("ctrl+k", "up"),
    ("enter", "confirm"),
    ("ctrl+c", "exit"),
    ("esc", "back"),
    ("f1", "help"),
]

ABOUT_KB = [
    ("ctrl+c", "exit"),
    ("esc", "back"),
    ("f1", "help"),
]

HELP_KB = [
    ("l", "next tab"),
    ("h", "previous tab"),
    ("j", "down"),
    ("k", "up"),
    ("esc", "back"),
]

CHAT_KB = [
    ("ctrl+j", "focus up"),
    ("ctrl+k", "focus down"),
    ("enter", "send message"),
    ("shift+enter", "new line"),
    ("ctrl+c", "exit"),
    ("esc", "menu"),
    ("f1", "help"),
]
